#include <stdio.h> 
#include "ldap.h"  

/*
 * This test program was done for ticket 47361 
 */

 /* Adjust these setting for your own LDAP server */ 
 #define HOSTNAME "localhost"
 #define PORT_NUMBER  2011
 #define FIND_DN  "dc=com" 
#define DIRMGRNAME "cn=directory manager" 
#define DIRMGRPWD "Secret123"
#define UID_BIND "uid=<entry>,dc=com"
#define UID_PWD "Secret123"
 int 
 do_it( int port)
 { 
   LDAP         *ld; 
   LDAPMessage  *result, *e; 
   BerElement   *ber; 
   char         *a; 
   char         **vals; 
   int          i, rc; 
   LDAPControl	dummy_ctl = {NULL};
   LDAPControl       *requestctrls[ 2 ];
   LDAPControl       *emptyctrls[ 1 ];

   /* Get a handle to an LDAP connection. */ 
   if ( (ld = ldap_init( (char*) HOSTNAME, port )) == NULL ) { 
     perror( "ldap_init" ); 
     return( 1 ); 
   } 

   /* Bind anonymously to the LDAP server. */ 
   rc = ldap_simple_bind_s( ld, DIRMGRNAME, DIRMGRPWD ); 
   if ( rc != LDAP_SUCCESS ) { 
     fprintf(stderr, "ldap_simple_bind_s: %s\n", ldap_err2string(rc)); 
     return( 1 ); 
   } 



#if 0
	dummy_ctl.ldctl_oid = "2.16.840.1.113730.3.4.2";
	dummy_ctl.ldctl_value.bv_val = NULL;
	dummy_ctl.ldctl_value.bv_len = 0;
	dummy_ctl.ldctl_iscritical = 1;
#endif
	requestctrls[0] = 0;
	requestctrls[1] = 0;
	emptyctrls[0] = 0;

   /* Search for the entry. */ 
   if ( ( rc = ldap_search_ext_s( ld, FIND_DN, LDAP_SCOPE_BASE, 
     "(objectclass=*)", NULL, 0, requestctrls, NULL, LDAP_NO_LIMIT, 
     LDAP_NO_LIMIT, &result ) ) != LDAP_SUCCESS ) { 
     fprintf(stderr, "ldap_search_ext_s: %s\n", ldap_err2string(rc)); 
     return( 1 ); 
   } 
   /* Since we are doing a base search, there should be only 
 *       one matching entry.  */ 
   e = ldap_first_entry( ld, result ); 
   if ( e != NULL ) { 
     printf( "\nFound %s:\n\n", FIND_DN ); 
     /* Iterate through each attribute in the entry. */ 
     for ( a = ldap_first_attribute( ld, e, &ber ); 
       a != NULL; a = ldap_next_attribute( ld, e, ber ) ) { 
       /* For each attribute, print the attribute name and values. */ 
       if ((vals = ldap_get_values( ld, e, a)) != NULL ) { 
         for ( i = 0; vals[i] != NULL; i++ ) { 
           printf( "%s: %s\n", a, vals[i] ); 
         } 
         ldap_value_free( vals ); 
       } 
       ldap_memfree( a ); 
     } 
     if ( ber != NULL ) { 
       ber_free( ber, 0 ); 
     } 
   } 
   ldap_msgfree( result ); 
   ldap_unbind( ld ); 
   return( 0 ); 
 }

int
main(int argc, char **argv)
{
	int i;
	int nb_ctls;
	int port;

	if (argc == 2) {
		port = atoi(argv[1]);
	} else {
		port = PORT_NUMBER;
	}
	do_it(port);
}
